//looks for all EEP files and checks the schema
const fs = require('fs');

async function generateEEPList() {
	const path = __dirname.replace('lib/devHelper', 'lib/definitions/eep');
	const folders = await fs.readdirSync(path);
	let warnings = 0;
	for(const d in folders) {
		const files = await fs.readdirSync(`${path}/${folders[d]}`);
		for (const f in files) {
			let result = {file: files[f]};
			const file = JSON.parse( fs.readFileSync(`${path}/${folders[d]}/${files[f]}`, {encoding: 'utf8'}) );
			//console.log(file.eep);
			if(!file.eep){
				result.eep = 'No EEP Attribute found';
			}else{
				const eep = file.eep.split('-');
				if(eep.length != 3){
					if(result.eep !== undefined){
						result.eep += ', Incorrect eep';
					}else{
						result.eep = 'Incorrect eep';
					}

				}
				const fileEEP = files[f].replace('.json', '');
				if(fileEEP !== file.eep){
					if(result.eep !== undefined){
						result.eep += ', EEP does not match file name';
					}else{
						result.eep = 'EEP does not match file name';
					}

				}

			}

			if(!file.rorg_number){
				result.rorg_number = 'No RORG number found';
			}

			if(!file.telegram){
				result.telegram = 'No telegram type found';
			}

			if(!file.func_number){
				result.func_number = 'No function number found';
			}

			if(!file.type_number){
				result.type_number = 'No type number found';
			}

			if(!file.type_title){
				result.type_title = 'No type title found';
			}

			if(!file.case){
				result.case = 'No case found';
			}

			if(file.bidirectional === undefined || file.bidirectional === null){
				result.bidirectional = 'Missing bidirectional flag. Run check again after adding the flag';
			}

			//check cases
			if(file.case && file.case.length === undefined){
				result.case = 'Undefined length of cases. Not an array?';
			} else {
				for(const c in file.case){
					const Case = file.case[c];

					if( (file.bidirectional || file.bidirectional === false) && (Case.send === undefined || Case.send === null)){
						if(!result[c]) result[c] = {};
						result[c].send = 'This attribute has to be defined. It indicates if the case will be sent or not. false = do not send, true = send if condition is true.';
					}
					/*console.log(JSON.stringify(result) + ' ' + c);
					if(Case.condition && !Case.condition.value){
						result[c].condition.value = 'A value has to be defined for a condition.';
					}*/

					//check the datafields
					if(Case.datafield === undefined || Case.datafield === null){
						if(!result[c]) result[c] = {};
						result[c].datafield = 'Datafield has to be defined.';
					} else {
						if(!result[c]) result[c] = {};
						result[c].datafield = await checkDatafield(Case.datafield);
						if(Object.keys(result[c].datafield).length === 0) {
							delete result[c];
						}
					}
				}
			}

			//console.log(file.case.length);

			if(Object.keys(result).length > 1){
				console.log(JSON.stringify(result));
				warnings = warnings + 1;
			}
		}

	}
	console.log(`${warnings} file with warnings`);
}

async function checkDatafield(data){
	let result = {};

	for(const d in data){
		result[d] = {};
		if(!data[d].data){
			result[d].data = 'Field data not found. This field is used as name for the object in ioBroker, if there is one.';
		}
		if(data[d].data && data[d].data === 'fixed parameter'){
			if(!data[d].description){
				result[d].description = 'Missing description for fixed parameter';
			}

		}

		if(!data[d].bitoffs && data[d].shortcut !== 'CMD'){
			result[d].bitoffs = 'Missing Bit offset.';
		}

		if(!data[d].bitsize && data[d].shortcut !== 'CMD'){
			result[d].bitsize = 'Missing Bit size.';
		}

		if( (data[d].value === undefined || data[d].value === null) && (data[d].value_out === undefined || data[d].value_out === null)){
			result[d].value = 'Missing value.';
		}

		if(!data[d].shortcut && data[d].data !== 'fixed parameter'){
			result[d].data = 'Missing shortcut OR this datfield is a fixed parameter.';
		}

		if(result[d].iobroker && !result[d].shortcut){
			result[d].shortcut = 'Missing shortcut';
		}

		if(Object.keys(result[d]).length === 0){
			delete result[d];
		}
	}
	return result;
}

generateEEPList();
